@@ -50,6 +50,12 @@ GType g_desktop_touchpad_click_method_get_type (void) G_GNUC_CONST;
 #define G_DESKTOP_TYPE_DESKTOP_TOUCHPAD_CLICK_METHOD (g_desktop_touchpad_click_method_get_type())
 GType g_desktop_device_send_events_get_type (void) G_GNUC_CONST;
 #define G_DESKTOP_TYPE_DESKTOP_DEVICE_SEND_EVENTS (g_desktop_device_send_events_get_type())
+GType g_desktop_tablet_mapping_get_type (void) G_GNUC_CONST;
+#define G_DESKTOP_TYPE_DESKTOP_TABLET_MAPPING (g_desktop_tablet_mapping_get_type())
+GType g_desktop_pad_button_action_get_type (void) G_GNUC_CONST;
+#define G_DESKTOP_TYPE_DESKTOP_PAD_BUTTON_ACTION (g_desktop_pad_button_action_get_type())
+GType g_desktop_stylus_button_action_get_type (void) G_GNUC_CONST;
+#define G_DESKTOP_TYPE_DESKTOP_STYLUS_BUTTON_ACTION (g_desktop_stylus_button_action_get_type())
 /* enumerations from "cc-background-item.h" */
 GType cc_background_item_flags_get_type (void) G_GNUC_CONST;
 #define G_DESKTOP_TYPE_BACKGROUND_ITEM_FLAGS (cc_background_item_flags_get_type())
