@@ -1462,6 +1462,7 @@ nm_client_ready_cb (GObject *source_object,
   g_signal_connect (priv->nm_client, "device-removed",
                     G_CALLBACK (nm_device_changed), self);
 
+  nm_client_state_changed (priv->nm_client, NULL, self);
   nm_device_changed (priv->nm_client, NULL, self);
 }
 
@@ -1934,6 +1935,8 @@ add_power_saving_section (CcPowerPanel *self)
   gtk_widget_show_all (widget);
 }
 
+#define NEVER 0
+
 static void
 update_automatic_suspend_label (CcPowerPanel *self)
 {
@@ -1949,25 +1952,30 @@ update_automatic_suspend_label (CcPowerPanel *self)
   ac_timeout = g_settings_get_int (priv->gsd_settings, "sleep-inactive-ac-timeout");
   battery_timeout = g_settings_get_int (priv->gsd_settings, "sleep-inactive-battery-timeout");
 
-  if (ac_action == GSD_POWER_ACTION_NOTHING)
-    ac_timeout = 0;
-  if (battery_action == GSD_POWER_ACTION_NOTHING)
-    battery_timeout = 0;
+  if (ac_timeout < 0)
+    g_warning ("Invalid negative timeout for 'sleep-inactive-ac-timeout': %d", ac_timeout);
+  if (battery_timeout < 0)
+    g_warning ("Invalid negative timeout for 'sleep-inactive-battery-timeout': %d", battery_timeout);
+
+  if (ac_action == GSD_POWER_ACTION_NOTHING || ac_timeout < 0)
+    ac_timeout = NEVER;
+  if (battery_action == GSD_POWER_ACTION_NOTHING || battery_timeout < 0)
+    battery_timeout = NEVER;
 
   if (priv->has_batteries)
     {
-      if (ac_timeout == 0 && battery_timeout == 0)
+      if (ac_timeout == NEVER && battery_timeout == NEVER)
         s = _("Off");
-      else if (ac_timeout == 0 && battery_timeout != 0)
+      else if (ac_timeout == NEVER && battery_timeout > 0)
         s = _("When on battery power");
-      else if (ac_timeout != 0 && battery_timeout == 0)
+      else if (ac_timeout > 0 && battery_timeout == NEVER)
         s = _("When plugged in");
       else
         s = _("On");
     }
   else
     {
-      if (ac_timeout == 0)
+      if (ac_timeout == NEVER)
         s = _("Off");
       else
         s = _("On");
@@ -2235,7 +2243,6 @@ add_suspend_and_power_off_section (CcPowerPanel *self)
       gtk_box_pack_start (GTK_BOX (box), sw, FALSE, TRUE, 0);
       gtk_container_add (GTK_CONTAINER (widget), row);
       gtk_size_group_add_widget (priv->row_sizegroup, row);
-      update_automatic_suspend_label (self);
 
       dialog = priv->automatic_suspend_dialog;
       g_signal_connect (dialog, "delete-event", G_CALLBACK (gtk_widget_hide_on_delete), NULL);
@@ -2272,6 +2279,7 @@ add_suspend_and_power_off_section (CcPowerPanel *self)
                               G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
       set_ac_battery_ui_mode (self);
+      update_automatic_suspend_label (self);
     }
 
   if (g_strcmp0 (priv->chassis_type, "vm") == 0 ||
